import React from 'react';
import PropTypes from 'prop-types';
import escapeStringRegExp from 'escape-string-regexp';

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

/**
 * `<Completion />` elements describe different data sources. Multiple can be
 * used in the same [`<AutoComplete />`][AutoComplete] component.
 */

var Completion =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Completion, _React$Component);

  function Completion() {
    return _React$Component.apply(this, arguments) || this;
  }

  var _proto = Completion.prototype;

  _proto.render = function render() {
    return null;
  };

  return Completion;
}(React.Component);
/* eslint-disable react/no-unused-prop-types */


Completion.propTypes = process.env.NODE_ENV !== "production" ? {
  /**
   * String that triggers this completion type.
   */
  trigger: PropTypes.oneOfType([PropTypes.string, PropTypes.instanceOf(RegExp)]).isRequired,

  /**
   * Minimum amount of characters typed before suggestions will be given.
   */
  minLength: PropTypes.number,

  /**
   * Regex to extract the current completion value from the input. Can also be
   * used to "validate" the current completion value, so no suggestions will
   * be provided if it's "invalid".
   *
   * Uses the first capture group as the value to be completed, or the full
   * match if there are no capture groups. For example:
   *  - `/.*(@.*?)$/` + "Hello @ReA" → "@ReA"
   *  - `/\w+$/` + "This is sp" → "sp"
   */
  regex: PropTypes.instanceOf(RegExp),

  /**
   * Function that renders a single suggestion.
   *
   * @param {Object} suggestion - Suggestion descriptor.
   * @param {string} suggestion.key - Unique key for the suggestion element.
   *     See [Dynamic Children](https://facebook.github.io/react/docs/multiple-components.html#dynamic-children)
   *     for details.
   * @param {*} suggestion.value - Completion value of this suggestion.
   * @param {boolean} suggestion.selected - Whether this suggestion is
   *     currently selected.
   * @param {function} suggestion.select - Autocomplete this suggestion.
   * @returns {ReactElement}
   */
  renderSuggestion: PropTypes.func,

  /**
   * Get an array of possible completions.
   *
   * @default Searches the `completions` prop.
   *
   * @param {string} matchingValue - Current value to be completed, as
   *     extracted using `props.regex`.
   * @param {Object} props - Props of this `<Completion />` element.
   * @returns {Array.<*>}
   */
  getCompletions: PropTypes.func,

  /**
   * Optional array of completion values. This can be used if all possible
   * completions are known beforehand. If provided, a default `getCompletions`
   * function that searches this array will be used.
   */
  completions: PropTypes.array,
  // eslint-disable-line react/forbid-prop-types

  /**
   * Transform a completion value to a string that will be inserted into the
   * input component. By default, uses ``` `${props.trigger}${value} ` ```.
   * (Note the space at the end! If you want to add a space once a completion
   * is inserted, add it here.)
   *
   * @param {*} value - Completion value.
   * @param {Object} props - Props of this `<Completion />` element.
   * @returns {string}
   */
  getText: PropTypes.func
} : {};
Completion.defaultProps = {
  minLength: 3,
  regex: null,
  renderSuggestion: null,
  getCompletions: function getCompletions(value, _ref) {
    var trigger = _ref.trigger,
        completions = _ref.completions;
    var compare = value.substr(trigger.length).toLowerCase();
    return completions.filter(function (completion) {
      return completion.substr(0, compare.length).toLowerCase() === compare;
    });
  },
  completions: [],
  getText: function getText(value, _ref2) {
    var trigger = _ref2.trigger;
    return "" + trigger + value + " ";
  }
};

function getRegex(_ref) {
  var regex = _ref.regex,
      trigger = _ref.trigger;

  if (regex) {
    return regex;
  }

  var t = escapeStringRegExp(trigger);
  return new RegExp(".*(" + t + ".*?)$");
}

function setCursor(input, position) {
  input.setSelectionRange(position, position);
}
/**
 *
 */


var AutoComplete =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(AutoComplete, _React$Component);

  function AutoComplete() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this.state = {
      open: false,
      value: _this.props.defaultValue || '',
      input: null,
      currentSuggestions: [],
      selectedSuggestion: 0
    };

    _this.handleChange = function (event) {
      var _this$props = _this.props,
          children = _this$props.children,
          limit = _this$props.limit;
      var _event$target = event.target,
          value = _event$target.value,
          selectionEnd = _event$target.selectionEnd;
      var completingValue = value.slice(0, selectionEnd);
      var completionTypes = React.Children.map(children, function (child) {
        return child.props;
      }).reduce(function (types, completionType) {
        var rx = getRegex(completionType);
        var match = rx.exec(completingValue);

        if (match) {
          var matchingValue = match[1] || match[0];

          if (matchingValue.length >= completionType.minLength) {
            types.push({
              type: completionType,
              regex: rx,
              matchingValue: matchingValue
            });
          }
        }

        return types;
      }, []);
      var currentSuggestions = completionTypes.reduce(function (available, childProps) {
        var type = childProps.type,
            matchingValue = childProps.matchingValue;
        var currentLimit = limit - available.length; // Don't even ask for more completions if we've already reached our max.

        if (currentLimit <= 0) {
          return available;
        }

        var completions = type.getCompletions(matchingValue, type).slice(0, currentLimit);
        return available.concat(completions.map(function (completion) {
          return _extends({
            completion: completion
          }, childProps);
        }));
      }, []);

      _this.setState({
        value: value,
        input: event.target,
        currentSuggestions: currentSuggestions,
        selectedSuggestion: 0
      });

      var inputProps = _this.props.inputProps;

      if (inputProps.onChange && !event.defaultPrevented) {
        inputProps.onChange(event);
      }

      _this.sendUpdate(value);
    };

    _this.handleKeyDown = function (event) {
      var _this$state = _this.state,
          selectedSuggestion = _this$state.selectedSuggestion,
          currentSuggestions = _this$state.currentSuggestions;

      if (event.key === 'ArrowUp') {
        event.preventDefault();

        if (currentSuggestions.length > 0) {
          var nextSuggestion = selectedSuggestion > 0 ? selectedSuggestion - 1 : currentSuggestions.length - 1;

          _this.setState({
            selectedSuggestion: nextSuggestion
          });
        }
      } else if (event.key === 'ArrowDown') {
        event.preventDefault();

        if (currentSuggestions.length > 0) {
          var _nextSuggestion = (selectedSuggestion + 1) % currentSuggestions.length;

          _this.setState({
            selectedSuggestion: _nextSuggestion
          });
        }
      } else if (event.key === 'Enter' || event.key === 'Tab') {
        if (currentSuggestions[selectedSuggestion]) {
          event.preventDefault();

          _this.select(selectedSuggestion, event.target);
        }
      } else if (event.key === 'ArrowLeft' || event.key === 'ArrowRight') {
        var target = event.target;
        setTimeout(function () {
          return _this.handleChange({
            target: target
          });
        }, 1);
      } // Bubble up manually if necessary


      var inputProps = _this.props.inputProps;

      if (inputProps.onKeyDown && !event.defaultPrevented) {
        inputProps.onKeyDown(event);
      }
    };

    _this.handleFocus = function (event) {
      _this.setState({
        open: true
      });

      var inputProps = _this.props.inputProps;

      if (inputProps.onFocus) {
        inputProps.onFocus(event);
      }
    };

    _this.handleBlur = function (event) {
      // Make sure the blur event is handled _after_ any possible suggestion click
      // events. Otherwise, the suggestions list might close before a click event
      // is registered, and click-completing would be impossible.
      // It's a bit of a hack and hopefully there is a better way!
      setTimeout(function () {
        _this.setState({
          open: false
        });

        var inputProps = _this.props.inputProps;

        if (inputProps.onBlur) {
          inputProps.onBlur(event);
        }
      }, 16);
    };

    _this.select = function (idx, input) {
      var _this$state2 = _this.state,
          currentSuggestions = _this$state2.currentSuggestions,
          value = _this$state2.value;
      var cursorPosition = input ? input.selectionEnd : value.length;
      var completion = currentSuggestions[idx];

      if (input) {
        var type = completion.type,
            matchingValue = completion.matchingValue;
        var before = value.slice(0, cursorPosition - matchingValue.length);
        var after = value.slice(cursorPosition);
        var insertValue = type.getText(completion.completion, type);
        var newValue = "" + before + insertValue + after;

        _this.setState({
          value: newValue,
          currentSuggestions: []
        }, function () {
          input.focus();
          setCursor(input, before.length + insertValue.length);
        });

        _this.sendUpdate(newValue);
      }
    };

    return _this;
  }

  var _proto = AutoComplete.prototype;

  _proto.sendUpdate = function sendUpdate(value) {
    this.props.onUpdate(value);
  };

  _proto.renderSuggestion = function renderSuggestion(_ref2, idx) {
    var _this2 = this;

    var type = _ref2.type,
        completion = _ref2.completion;
    var _this$state3 = this.state,
        input = _this$state3.input,
        selectedSuggestion = _this$state3.selectedSuggestion; // Use the Completion type's suggestion renderer, or the default if it
    // doesn't have a custom one.

    var render = type.renderSuggestion || this.props.renderSuggestion;
    return render({
      key: "" + idx,
      value: completion,
      selected: idx === selectedSuggestion,
      select: function select() {
        return _this2.select(idx, input);
      }
    });
  };

  _proto.renderSuggestions = function renderSuggestions() {
    var renderSuggestions = this.props.renderSuggestions;
    var currentSuggestions = this.state.currentSuggestions;
    var suggestions = currentSuggestions.map(this.renderSuggestion, this);
    return renderSuggestions(suggestions);
  };

  _proto.render = function render() {
    var _this$state4 = this.state,
        open = _this$state4.open,
        currentSuggestions = _this$state4.currentSuggestions;
    var _this$props2 = this.props,
        InputComponent = _this$props2.inputComponent,
        inputProps = _this$props2.inputProps;
    var value = 'value' in this.props ? this.props.value : this.state.value;
    return React.createElement("span", null, React.createElement(InputComponent, _extends({}, inputProps, {
      value: value,
      onChange: this.handleChange,
      onKeyDown: this.handleKeyDown,
      onFocus: this.handleFocus,
      onBlur: this.handleBlur
    })), open && currentSuggestions.length > 0 ? this.renderSuggestions() : null);
  };

  return AutoComplete;
}(React.Component);

AutoComplete.defaultProps = {
  inputComponent: 'input',
  inputProps: {
    type: 'text'
  },
  renderSuggestion: function renderSuggestion(_ref3) {
    var key = _ref3.key,
        value = _ref3.value,
        selected = _ref3.selected,
        select = _ref3.select;
    return (// eslint-disable-next-line jsx-a11y/no-static-element-interactions
      React.createElement("div", {
        key: key,
        style: {
          fontWeight: selected ? 'bold' : 'normal'
        },
        onClick: select,
        onKeyDown: function onKeyDown(event) {
          return event.key === 'Enter' && select();
        }
      }, value)
    );
  },
  renderSuggestions: function renderSuggestions(suggestions) {
    return React.createElement("div", null, suggestions);
  },
  children: [],
  limit: 15,
  defaultValue: '',
  onUpdate: function onUpdate() {}
};
AutoComplete.propTypes = process.env.NODE_ENV !== "production" ? {
  /**
   * Component to use for rendering the input element. Uses native `<input />`
   * by default.
   *
   * The component should accept `value`, `onChange` and `onKeyDown` props.
   */
  inputComponent: PropTypes.oneOfType([PropTypes.string, PropTypes.func]),

  /**
   * Props to pass to the input component.
   */
  inputProps: PropTypes.object,
  // eslint-disable-line react/forbid-prop-types

  /**
   * Function that renders a single suggestion. This can be overridden for
   * individual Completion types, in case they need custom rendering.
   *
   * @default `<div key={key} onClick={select}>{value}</div>`
   *
   * @param {Object} suggestion - Suggestion descriptor.
   * @param {string} suggestion.key - Unique key for the suggestion element.
   *     See [Dynamic Children](https://facebook.github.io/react/docs/multiple-components.html#dynamic-children)
   *     for details.
   * @param {*} suggestion.value - Completion value of this suggestion.
   * @param {boolean} suggestion.selected - Whether this suggestion is
   *     currently selected.
   * @param {function} suggestion.select - Autocomplete this suggestion.
   * @returns {ReactElement}
   */
  renderSuggestion: PropTypes.func,

  /**
   * Function that renders the suggestions list.
   *
   * @default `<div>{suggestions}</div>`
   *
   * @param {Array} suggestions - Array of children rendered by
   *     `renderSuggestion`.
   * @returns {ReactElement}
   */
  renderSuggestions: PropTypes.func,

  /**
   * Completion types as [`<Completion />`][Completion] elements.
   */
  children: PropTypes.node,

  /**
   * The maximum amount of suggestions to show.
   */
  limit: PropTypes.number,

  /**
   * Current string value of the input component. Optional, useful for
   * controlled inputs. Passed down to the input component as the value prop.
   */
  value: PropTypes.string,
  // eslint-disable-line react/require-default-props

  /**
   * Initial string value for uncontrolled inputs.
   */
  defaultValue: PropTypes.string,

  /**
   * Fired when the input component's value changes. Use this for controlled
   * inputs.
   *
   * @param {string} newValue
   */
  onUpdate: PropTypes.func
} : {};

export default AutoComplete;
export { Completion };
//# sourceMappingURL=react-abstract-autocomplete.es.js.map
